#  ncgmp09_TranslateToShp.py
#
#  Converts an ncgmp09-style ArcGIS geodatabase to open file formats
#    (shape files, .dbf files, and pipe-delimited text files)
#    without loss of any information.  Field renaming is documented in 
#    output file logfile.txt
#
#  Ralph Haugerud, USGS, Seattle
#    rhaugerud@usgs.gov

import arcgisscripting, sys, os, glob

versionString = 'version of 28 September 2009'
Is93 = True

def usage():
	print """USAGE: ncgmp09_TranslateToShp.py  <geodatabase> <outputWorkspace>
  where <geodatabase> must be an existing ArcGIS geodatabase.
  <geodatabase> may be a personal or file geodatabase, and the 
  .gdb or .mdb extension must be included.
  Output is written to directory <geodatabase (no extension)>-shp 
  in <outputWorkspace>. The output directory, if it already exists,
  will be overwritten.
"""

def addMsgAndPrint(gp,msg, severity=0): 
	# prints msg to screen and adds msg to the geoprocessor (in case this is run as a tool) 
	# print msg 
	try: 
	  for string in msg.split('\n'): 
		# Add appropriate geoprocessing message 
		if severity == 0: 
			gp.AddMessage(string) 
		elif severity == 1: 
			gp.AddWarning(string) 
		elif severity == 2: 
			gp.AddError(string) 
	except: 
		pass 

def remapFieldName(name):
	name2 = name.replace('And','')
	name2 = name2.replace('Of','')
	name2 = name2.replace('Unit','Un')
	name2 = name2.replace('Source','Src')
	name2 = name2.replace('Shape','Shp')
	newName = ''
	for i in range(0,len(name2)):
		if name2[i] == name2[i].upper():
			newName = newName + name2[i]
			j = 1
		else:
			j = j+1
			if j < 4:
				newName = newName + name2[i]
	if len(newName) > 10:
		if newName[1:3] == newName[1:3].lower():
			newName = newName[0]+newName[3:]
	if len(newName) > 10:
		if newName[3:5] == newName[3:5].lower():
			newName = newName[0:2]+newName[5:]
	if len(newName) > 10:
		print name, newName
	return newName	

def dumpTable(gp,fc,outName,isSpatial,outputDir,logfile):
	if isSpatial:
		logfile.write('  feature class '+fc+' dumped to shapefile '+outName+'\n')
	else:
		logfile.write('  table '+fc+' dumped to table '+outName+'\n')
	logfile.write('    field name remapping: \n')
	# describe
	fields = gp.ListFields(fc)
	longFields = []
	shortFieldName = {}
	for field in fields:
		# translate field names
		if len(field.Name) > 10:
			shortFieldName[field.Name] = remapFieldName(field.Name)
			# write field name translation to logfile
			logfile.write('      '+field.Name+' > '+shortFieldName[field.Name]+'\n')
		else:
			shortFieldName[field.Name] = field.Name
		if field.Length > 255:
			longFields.append(field.Name)
	# setup field mapping
	fms = gp.CreateObject("FieldMappings")
	# and add the table to generate a field map object for each field
	fms.AddTable(fc)
	i = 0
	while i < fms.fieldcount:
		# one fieldmap object per field
		fm = fms.getfieldmap(i)
		# get the output field object, really a copy of it
		fldout = fm.outputfield
		# set the name for the output field through the lookup dictionary
		fldout.name = shortFieldName[fm.getinputfieldname(0)]
		# re-apply the field to the fieldmap object
		fm.outputfield = fldout
		# and replace the existing field map at index i with the updated one.
		fms.replacefieldmap(i, fm)
		i = i + 1
	# export to shapefile (named prefix+name)
	if isSpatial:
		gp.FeatureClassToFeatureClass_conversion(fc,outputDir,outName,'#',fms)
	else:
		gp.TableToTable_conversion(fc,outputDir,outName,'#',fms)
	# if any field lengths > 255, write csv file of _IDs and long fields
	if len(longFields) > 0:
		csvFile = open(outputDir+'/'+outName[0:-4]+'.txt','w')
		csvFile.write(fc+'_ID')
		for field in longFields:
			csvFile.write(','+field)
		csvFile.write('\n')
		rows = gp.searchCursor(fc)
		row = rows.next()
		while row:
			rowString = row.GetValue(fc+'_ID')
			for field in longFields:
				if row.GetValue(field) <> None:
					rowString = rowString+'|'+row.GetValue(field)
				else:
					rowString = rowString+'|'
			try:
				csvFile.write(rowString+'\n')	
			except:
				rStr = ''
				for i in range(len(rowString)):
				   if ord(rowString[i]) < 128:
					rStr = rStr + rowString[i]
				   else: 
					rStr = rStr + '<chr('+str(ord(rowString[i]))+')>'
				csvFile.write(rStr+'\n')
			row = rows.next()
		csvFile.close()

def main(gp,gdb,outWS):
	outputDir = outWS+'/'+os.path.basename(gdb)[0:-4]+'-shp'
	absOutputDir = os.path.abspath(outputDir)
	addMsgAndPrint(gp,'  Making '+absOutputDir+'/...')
	if os.path.exists(outputDir):
		if os.path.exists(outputDir+'/info'):
			for fl in glob.glob(outputDir+'/info/*'):
				os.remove(fl)
			os.rmdir(outputDir+'/info')
		for fl in glob.glob(outputDir+'/*'):
			os.remove(fl)
		os.rmdir(outputDir)
	os.mkdir(outputDir)
	logfile = open(outputDir+'/logfile.txt','w')
	logfile.write('file written by ncgmp_TranslateToShp.py, '+versionString+'\n')
	
	gp.workspace = gdb
	gp.overwriteoutput = True
	# list featuredatasets
	fds = gp.ListDataSets()
	# for each featuredataset
	for fd in fds:
		addMsgAndPrint(gp, '  Processing feature data set '+fd+'...')
		logfile.write('Feature data set '+fd+' \n')
		spatialRef = gp.Describe(fd).SpatialReference
		logfile.write('  spatial reference framework\n')
		logfile.write('    name = '+spatialRef.Name+'\n')
		logfile.write('    spheroid = '+spatialRef.SpheroidName+'\n')
		logfile.write('    projection = '+spatialRef.ProjectionName+'\n')
		logfile.write('    units = '+spatialRef.LinearUnitName+'\n')
		# generate featuredataset prefix
		pfx = ''
		for i in range(0,len(fd)-1):
			if fd[i] == fd[i].upper():
				pfx = pfx + fd[i]
		# for each featureclass in dataset
		gp.workspace = gdb
		gp.workspace = fd
		for fc in gp.ListFeatureClasses():
			outName = pfx+'_'+fc+'.shp'
			addMsgAndPrint(gp, '    dumping table '+outName+'...')
			dumpTable(gp,fc,outName,True,absOutputDir,logfile)
		logfile.write('\n')
	# list tables
	gp.workspace = gdb
	for tbl in gp.listTables():
		addMsgAndPrint(gp, '  dumping table '+tbl+'...')
		outName = tbl+'.dbf'
		dumpTable(gp,tbl,outName,False,absOutputDir,logfile)
	logfile.close()

### START HERE ###
print '  '+versionString
print '  checking input'
if len(sys.argv) <> 3 or not os.path.exists(sys.argv[1]) or not os.path.exists(sys.argv[2]):
	usage()
else:
	try:
		print '  loading arcprocessor'
		if Is93:
			gp = arcgisscripting.create(9.3)
		else:
			gp = arcgisscripting.create()
	except:
		addMsgAndPrint(gp,'  Unable to create geoprocessing object. Arc license not present??')
	else:
		main(gp,os.path.abspath(sys.argv[1]),sys.argv[2])

